#! /vendor/bin/sh

if [[ -f /system/bin/swapon ]]; then
    alias swapon="/system/bin/swapon"
    alias swapoff="/system/bin/swapoff"
    alias mkswap="/system/bin/mkswap"
elif [[ -f /vendor/bin/swapon ]]; then
    alias swapon="/vendor/bin/swapon"
    alias swapoff="/vendor/bin/swapoff"
    alias mkswap="/vendor/bin/mkswap"
fi

if [[ -f /system/bin/dd ]]; then
    alias dd="/system/bin/dd"
elif [[ -f /vendor/bin/dd ]]; then
    alias dd="/vendor/bin/dd"
fi

swap_config="/data/swap_config.conf"
function read_property()
{
    cat $swap_config | grep -v '^#' | grep "^$1=" | cut -f2 -d '='
}

# set_value value path
function set_value()
{
    if [[ -f $2 ]]; then
      echo $1 > $2
    fi
}

# read_property prop
# 解析配置
allow_swap=$(read_property swap)
config_swap_size=$(read_property swap_size)
config_swap_priority=$(read_property swap_priority)
config_swap_use_loop=$(read_property swap_use_loop)
allow_zram=$(read_property zram)
config_zram_size=$(read_property zram_size)
config_swappiness=$(read_property swappiness)
config_extra_free_kbytes=$(read_property extra_free_kbytes)
# config_auto_lmk=$(read_property auto_lmk)
config_comp_algorithm=$(read_property comp_algorithm)

swapdir="/data"
swapfile="${swapdir}/swapfile"
recreate="${swapdir}/swap_recreate"

loop_save="vtools.swap.loop"
next_loop_path=""
# 获取下一个loop设备的索引
function get_next_loop() {
    local current_loop=`getprop $loop_save`

    if [[ "$current_loop" != "" ]]
    then
        next_loop_path="$current_loop"
    fi

    local loop_index=0
    # local used=`blkid | grep /dev/block/loop | cut -f1 -d ":"`
    local used=`blkid | grep /dev/block/loop`
    for loop in /dev/block/loop*
    do
        if [[ "$loop_index" -gt "0" ]]
        then
            if [[ `echo $used | grep /dev/block/loop$loop_index` = "" ]]
            then
                return $loop_index
            fi
        fi
        local loop_index=`expr $loop_index + 1`
    done

    if [[ -e "/dev/block/loop$loop_index" ]]; then
        next_loop_path="/dev/block/loop$loop_index"
    else
        next_loop_path=""
    fi
}

function configure_swap() {
    echo "设置SWAP"

    set_value 1 /proc/sys/vm/swap_ratio_enable
    set_value 90 /proc/sys/vm/swap_ratio

    # mkdir -p ${swapdir}

    # 判断是否需要重新创建文件
    if [[ -f ${recreate} ]]; then
        rm -rf ${swapfile}*
        rm -f ${recreate}
        echo "    删除已有swapfile"
    fi

    # 是否已经创建文件
    if [[ ! -f ${swapfile} ]]; then
        dd if=/dev/zero of=${swapfile} bs=1m count=${config_swap_size}
        echo "    创建swapfile"
    fi

    # 记录挂载点
    swap_mount=$swapfile

    # 如果需要挂载为回环设备，则先挂载并记录挂载点参数
    if [[ $config_swap_use_loop == "true" ]]; then
        echo "    获取下一个可用回环设备"
        get_next_loop

        if [[ "$next_loop_path" != "" ]]; then
            swap_mount=$next_loop_path
        else
            echo '  所有回环设备都已被占用，SWAP无法完成挂载！' 1>&2
            return
        fi

        # losetup $swap_mount $swapfile # 挂载
        if [[ -e $swap_mount ]]
        then
            echo "   删除已有回环设备"
            losetup -d $swap_mount      # 删除loop设备
        fi

        echo "    挂载回环设备"
        losetup $swap_mount $swapfile   # 挂载为loop设备
        setprop $loop_save $next_loop_path
    fi

    echo "    初始化swapfile"
    mkswap ${swap_mount}

    echo "    开启SWAP"
    # 判断是否自定义优先级
    if [[ "$config_swap_priority" != "" ]] && [[ "$config_swap_priority" -gt -1 ]]; then
        swapon ${swap_mount} -p $config_swap_priority
    else
        swapon ${swap_mount}
    fi
}

function configure_zram() {
    echo "设置ZRAM"

    current_disksize=`cat /sys/block/zram0/disksize`
    target_disksize="${config_zram_size}m" # 2001M(2098200576Bytes)

    echo "    重置ZRAM"
    swapoff /dev/block/zram0 2>/dev/null
    echo 1 > /sys/block/zram0/reset

    echo "    设置ZRAM压缩方式"
    if [[ "$config_comp_algorithm" != "" ]]; then
        echo $config_comp_algorithm > /sys/block/zram0/comp_algorithm
    fi

    set_value 4 /sys/block/zram0/max_comp_streams

    echo "    设置ZRAM大小"
    echo $target_disksize > /sys/block/zram0/disksize

    echo "    初始化ZRAM"
    mkswap /dev/block/zram0

    echo "    启动ZRAM"
    if [[ "$config_swap_priority" != "" ]]; then
        if [[ "$config_swap_priority" == "0" ]]; then
            swapon /dev/block/zram0 -p 0
        else
            swapon /dev/block/zram0
        fi
    else
        swapon /dev/block/zram0
    fi
}

echo "设置基本参数"
set_value '12800,16384,18432,20480,24576,32768' /sys/module/lowmemorykiller/parameters/minfree
set_value 53059 /sys/module/lowmemorykiller/parameters/vmpressure_file_min
set_value 0 /sys/module/lowmemorykiller/parameters/enable_adaptive_lmk
set_value 1 /sys/module/lowmemorykiller/parameters/oom_reaper
set_value 0 /sys/module/vmpressure/parameters/allocstall_threshold

# 相比默认设置，降低了读写缓冲区大小
# 会导致IO性能略微下降，但能增加一点点可用RAM
set_value 3 /proc/sys/vm/dirty_background_ratio
set_value 8 /proc/sys/vm/dirty_ratio
set_value 1000 /proc/sys/vm/dirty_expire_centisecs
set_value 3000 /proc/sys/vm/dirty_writeback_centisecs
set_value 128 /sys/block/sda/queue/read_ahead_kb

# ZRAM、SWAP
if [[ $allow_zram == "true" ]] && [[ "$config_zram_size" != "" ]]; then
    configure_zram
else
    echo "关闭ZRAM"
    swapoff /dev/block/zram0 2>/dev/null
fi

if [[ $allow_swap == "true" ]] && ([[ "$config_swap_size" != "" ]] || [[ -f ${swapfile} ]]); then
    configure_swap
fi

echo "即将完成..."
set_value 8192 /sys/module/process_reclaim/parameters/per_swap_size
set_value 80 /sys/module/process_reclaim/parameters/pressure_min
set_value 90 /sys/module/process_reclaim/parameters/pressure_max
set_value 0 /sys/module/process_reclaim/parameters/enable_process_reclaim

path='/proc/sys/vm/extra_free_kbytes'
# 设置min_free_kbytes
if [[ "$config_extra_free_kbytes" != "" ]]; then
    echo ""
    echo "设置min_free_kbytes"
    chmod 644 /proc/sys/vm/min_free_kbytes 2>/dev/null
    set_value "$config_extra_free_kbytes" $path
fi

function force_compact()
{
    echo ""
    echo "强制压缩内存"

    ratio=50
    min_free_kb=262144

    echo "    清空缓存"
    echo 3 > /proc/sys/vm/drop_caches
    echo 1 > /proc/sys/vm/compact_memory

    # /proc/sys/vm/min_free_kbytes 触发的内存回收过程会卡死界面
    # 而extra_free_kbytes则只会在后台回收内存
    min_free_kb=`cat $path`

    MemStr=`cat /proc/meminfo | grep MemTotal`
    Mem=${MemStr:16:8}

    echo $(($Mem / 100 * $ratio)) > $path

    time=20
    echo '    持续回收内存 ' $time '秒'
    sleep $time
    # 内存回收完毕，还原extra_free_kbytes
    set_value "$min_free_kb" "$path"
}

echo ""
if [[ "$config_swappiness" != "" ]]; then
    echo "设置swappiness > $config_swappiness"
    echo $config_swappiness > /proc/sys/vm/swappiness
fi
